﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Numerics;
using System.Runtime.InteropServices.WindowsRuntime;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Composition;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Hosting;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Media.Animation;
using Windows.UI.Xaml.Navigation;

// The Blank Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=402352&clcid=0x409

namespace Cancel2016
{
    /// <summary>
    /// An empty page that can be used on its own or navigated to within a Frame.
    /// </summary>
    public sealed partial class MainPage : Page
    {
        private Compositor _compositor;
        private Visual _buttonVisual;
        public MainPage()
        {
            this.InitializeComponent();
            Loaded += (s, e) =>
            {
                _buttonVisual = ElementCompositionPreview.GetElementVisual(CompositionButton);
                _compositor = _buttonVisual.Compositor;

                var position = CompositionButton.TransformToVisual(this).TransformPoint(new Point(0, 0));
                _buttonVisual.Offset = new Vector3((float)position.X, (float)position.Y, 0);

            };
        }

        private void StartStoryboard(object sender, RoutedEventArgs e)
        {
            var transform = new TranslateTransform();
            StoryboardButton.RenderTransform = transform;

            var sb = new Storyboard();
            sb.Duration = TimeSpan.FromSeconds(1.5);
            sb.RepeatBehavior = RepeatBehavior.Forever;
            sb.AutoReverse = true;

            var easingFunction = new CubicEase {EasingMode = EasingMode.EaseInOut};

            var animation = new DoubleAnimationUsingKeyFrames();
            animation.KeyFrames.Add(new EasingDoubleKeyFrame
            {
                KeyTime =new KeyTime(),
                Value = 0,
                EasingFunction = easingFunction
            });
            animation.KeyFrames.Add(new EasingDoubleKeyFrame
            {
                KeyTime = KeyTime.FromTimeSpan(sb.Duration.TimeSpan),
                Value = 50,
                EasingFunction = easingFunction
            });
            animation.KeyFrames.Add(new EasingDoubleKeyFrame
            {
                KeyTime = KeyTime.FromTimeSpan(sb.Duration.TimeSpan + sb.Duration.TimeSpan),
                Value = 0,
                EasingFunction = easingFunction
            });

            Storyboard.SetTarget(animation, transform);
            Storyboard.SetTargetProperty(animation, nameof(transform.Y));
            sb.Children.Add(animation);

            sb.Begin();
        }

        private void StartAnimation(object sender, RoutedEventArgs e)
        {
            var position = _buttonVisual.Offset;
            var offsetAnimation = _compositor.CreateVector3KeyFrameAnimation();
            offsetAnimation.InsertKeyFrame(0f, new Vector3(position.X, position.Y, 0f));
            offsetAnimation.InsertKeyFrame(0.5f, new Vector3(position.X, position.Y + 50, 0f));
            offsetAnimation.InsertKeyFrame(1f, new Vector3(position.X, position.Y, 0f));

            offsetAnimation.Duration = TimeSpan.FromSeconds(3);
            offsetAnimation.IterationBehavior = AnimationIterationBehavior.Forever;
            _buttonVisual.StartAnimation(nameof(Visual.Offset), offsetAnimation);

            var opacityAnimation = _compositor.CreateScalarKeyFrameAnimation();
            opacityAnimation.InsertKeyFrame(0f, 1f);
            opacityAnimation.InsertKeyFrame(.5f, .1f);
            opacityAnimation.InsertKeyFrame(1f, 1f);
            opacityAnimation.Duration = TimeSpan.FromSeconds(3);
            opacityAnimation.IterationBehavior = AnimationIterationBehavior.Forever;
            _buttonVisual.StartAnimation(nameof(Visual.Opacity), opacityAnimation);

            var linear = _compositor.CreateLinearEasingFunction();

            var rotationAnimation = _compositor.CreateScalarKeyFrameAnimation();
            _buttonVisual.CenterPoint = new Vector3((float) CompositionButton.ActualWidth/2,
                (float) CompositionButton.ActualHeight/2, 0);
            rotationAnimation.InsertKeyFrame(0f, 0f, linear);
            rotationAnimation.InsertKeyFrame(.5f, 360f, linear);
            rotationAnimation.InsertKeyFrame(1f, 720f, linear);
            rotationAnimation.Duration = TimeSpan.FromSeconds(3);
            rotationAnimation.IterationBehavior = AnimationIterationBehavior.Forever;
            _buttonVisual.StartAnimation(nameof(Visual.RotationAngleInDegrees), rotationAnimation);

            var offset = _buttonVisual.Offset;
            var expressionAnimation = _compositor.CreateExpressionAnimation("clamp(Y + thumb.Offset.X, Y, Y+100)");

            var thumb = Slider.FindVisualChild<Thumb>();
            var thumbVisual = ElementCompositionPreview.GetElementVisual(thumb);

            expressionAnimation.SetReferenceParameter("thumb", thumbVisual);
            expressionAnimation.SetScalarParameter("Y", offset.Y);

            _buttonVisual.StartAnimation("Offset.Y", expressionAnimation);
        }
    }

    public static class DependencyObjectExtensions
    {
        public static T FindVisualChild<T>(this DependencyObject obj)
            where T : DependencyObject
        {
            for (int i = 0; i < VisualTreeHelper.GetChildrenCount(obj); i++)
            {
                var child = VisualTreeHelper.GetChild(obj, i);
                if (child != null && child is T) return (T)child;

                T childOfChild = FindVisualChild<T>(child);
                if (childOfChild != null) return childOfChild;
            }
            return null;
        }


        public static T FindVisualParent<T>(this DependencyObject obj)
        where T : DependencyObject
        {
            var parent = VisualTreeHelper.GetParent(obj);
            if (parent == default(T))
            {
                return default(T);
            }

            if (parent is T) return (T)parent;
            return FindVisualParent<T>(parent);
        }
    }
}
